@@ -6,14 +6,21 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-import time
-import random
-
+import datetime
+import gzip
+import itertools
+import io
 import json
+import random
 import socket
+import struct
+import time
+from itertools import chain, islice
+
+import msgpack
 import requests
 import requests.exceptions
-from six.moves import xrange
+from requests.adapters import HTTPAdapter
 from six.moves.urllib.parse import urlparse
 
 from influxdb.line_protocol import make_lines, quote_ident, quote_literal
@@ -29,6 +36,9 @@ class InfluxDBClient(object):
     connect to InfluxDB. Requests can be made to InfluxDB directly through
     the client.
 
+    The client supports the use as a `context manager
+    <https://docs.python.org/3/reference/datamodel.html#context-managers>`_.
+
     :param host: hostname to connect to InfluxDB, defaults to 'localhost'
     :type host: str
     :param port: port to connect to InfluxDB, defaults to 8086
@@ -50,8 +60,12 @@ class InfluxDBClient(object):
     :param timeout: number of seconds Requests will wait for your client to
         establish a connection, defaults to None
     :type timeout: int
-    :param retries: number of retries your client will try before aborting,
-        defaults to 3. 0 indicates try until success
+    :param retries: number of attempts your client will make before aborting,
+        defaults to 3
+        0 - try until success
+        1 - attempt only once (without retry)
+        2 - maximum two attempts (including one retry)
+        3 - maximum three attempts (default option)
     :type retries: int
     :param use_udp: use UDP to connect to InfluxDB, defaults to False
     :type use_udp: bool
@@ -66,6 +80,18 @@ class InfluxDBClient(object):
         as a single file containing the private key and the certificate, or as
         a tuple of both files’ paths, defaults to None
     :type cert: str
+    :param gzip: use gzip content encoding to compress requests
+    :type gzip: bool
+    :param session: allow for the new client request to use an existing
+        requests Session, defaults to None
+    :type session: requests.Session
+    :param headers: headers to add to Requests, will add 'Content-Type'
+        and 'Accept' unless these are already present, defaults to {}
+    :type headers: dict
+    :param socket_options: use custom tcp socket options,
+        If not specified, then defaults are loaded from
+        ``HTTPConnection.default_socket_options``
+    :type socket_options: list
 
     :raises ValueError: if cert is provided but ssl is disabled (set to False)
     """
@@ -86,6 +112,10 @@ class InfluxDBClient(object):
                  pool_size=10,
                  path='',
                  cert=None,
+                 gzip=False,
+                 session=None,
+                 headers=None,
+                 socket_options=None,
                  ):
         """Construct a new InfluxDBClient object."""
         self.__host = host
@@ -99,11 +129,16 @@ class InfluxDBClient(object):
         self._verify_ssl = verify_ssl
 
         self.__use_udp = use_udp
-        self.__udp_port = udp_port
-        self._session = requests.Session()
-        adapter = requests.adapters.HTTPAdapter(
+        self.__udp_port = int(udp_port)
+
+        if not session:
+            session = requests.Session()
+
+        self._session = session
+        adapter = _SocketOptionsAdapter(
             pool_connections=int(pool_size),
-            pool_maxsize=int(pool_size)
+            pool_maxsize=int(pool_size),
+            socket_options=socket_options
         )
 
         if use_udp:
@@ -142,10 +177,21 @@ class InfluxDBClient(object):
             self._port,
             self._path)
 
-        self._headers = {
-            'Content-Type': 'application/json',
-            'Accept': 'text/plain'
-        }
+        if headers is None:
+            headers = {}
+        headers.setdefault('Content-Type', 'application/json')
+        headers.setdefault('Accept', 'application/x-msgpack')
+        self._headers = headers
+
+        self._gzip = gzip
+
+    def __enter__(self):
+        """Enter function as used by context manager."""
+        return self
+
+    def __exit__(self, _exc_type, _exc_value, _traceback):
+        """Exit function as used by context manager."""
+        self.close()
 
     @property
     def _baseurl(self):
@@ -231,7 +277,7 @@ class InfluxDBClient(object):
         self._username = username
         self._password = password
 
-    def request(self, url, method='GET', params=None, data=None,
+    def request(self, url, method='GET', params=None, data=None, stream=False,
                 expected_response_code=200, headers=None):
         """Make a HTTP request to the InfluxDB API.
 
@@ -243,6 +289,8 @@ class InfluxDBClient(object):
         :type params: dict
         :param data: the data of the request, defaults to None
         :type data: str
+        :param stream: True if a query uses chunked responses
+        :type stream: bool
         :param expected_response_code: the expected response code of
             the request, defaults to 200
         :type expected_response_code: int
@@ -266,17 +314,39 @@ class InfluxDBClient(object):
         if isinstance(data, (dict, list)):
             data = json.dumps(data)
 
+        if self._gzip:
+            # Receive and send compressed data
+            headers.update({
+                'Accept-Encoding': 'gzip',
+                'Content-Encoding': 'gzip',
+            })
+            if data is not None:
+                # For Py 2.7 compatability use Gzipfile
+                compressed = io.BytesIO()
+                with gzip.GzipFile(
+                    compresslevel=9,
+                    fileobj=compressed,
+                    mode='w'
+                ) as f:
+                    f.write(data)
+                data = compressed.getvalue()
+
         # Try to send the request more than once by default (see #103)
         retry = True
         _try = 0
         while retry:
             try:
+                if "Authorization" in headers:
+                    auth = (None, None)
+                else:
+                    auth = (self._username, self._password)
                 response = self._session.request(
                     method=method,
                     url=url,
-                    auth=(self._username, self._password),
+                    auth=auth if None not in auth else None,
                     params=params,
                     data=data,
+                    stream=stream,
                     headers=headers,
                     proxies=self._proxies,
                     verify=self._verify_ssl,
@@ -289,17 +359,34 @@ class InfluxDBClient(object):
                 _try += 1
                 if self._retries != 0:
                     retry = _try < self._retries
-                if method == "POST":
-                    time.sleep((2 ** _try) * random.random() / 100.0)
                 if not retry:
                     raise
+                if method == "POST":
+                    time.sleep((2 ** _try) * random.random() / 100.0)
+
+        type_header = response.headers and response.headers.get("Content-Type")
+        if type_header == "application/x-msgpack" and response.content:
+            response._msgpack = msgpack.unpackb(
+                packed=response.content,
+                ext_hook=_msgpack_parse_hook,
+                raw=False)
+        else:
+            response._msgpack = None
+
+        def reformat_error(response):
+            if response._msgpack:
+                return json.dumps(response._msgpack, separators=(',', ':'))
+            else:
+                return response.content
+
         # if there's not an error, there must have been a successful response
         if 500 <= response.status_code < 600:
-            raise InfluxDBServerError(response.content)
+            raise InfluxDBServerError(reformat_error(response))
         elif response.status_code == expected_response_code:
             return response
         else:
-            raise InfluxDBClientError(response.content, response.status_code)
+            err_msg = reformat_error(response)
+            raise InfluxDBClientError(err_msg, response.status_code)
 
     def write(self, data, params=None, expected_response_code=204,
               protocol='json'):
@@ -319,7 +406,7 @@ class InfluxDBClient(object):
         :returns: True, if the write operation is successful
         :rtype: bool
         """
-        headers = self._headers
+        headers = self._headers.copy()
         headers['Content-Type'] = 'application/octet-stream'
 
         if params:
@@ -346,17 +433,17 @@ class InfluxDBClient(object):
 
     @staticmethod
     def _read_chunked_response(response, raise_errors=True):
-        result_set = {}
         for line in response.iter_lines():
             if isinstance(line, bytes):
                 line = line.decode('utf-8')
             data = json.loads(line)
+            result_set = {}
             for result in data.get('results', []):
                 for _key in result:
                     if isinstance(result[_key], list):
                         result_set.setdefault(
                             _key, []).extend(result[_key])
-        return ResultSet(result_set, raise_errors=raise_errors)
+            yield ResultSet(result_set, raise_errors=raise_errors)
 
     def query(self,
               query,
@@ -447,13 +534,15 @@ class InfluxDBClient(object):
             method=method,
             params=params,
             data=None,
+            stream=chunked,
             expected_response_code=expected_response_code
         )
 
-        if chunked:
-            return self._read_chunked_response(response)
-
-        data = response.json()
+        data = response._msgpack
+        if not data:
+            if chunked:
+                return self._read_chunked_response(response)
+            data = response.json()
 
         results = [
             ResultSet(result, raise_errors=raise_errors)
@@ -544,8 +633,17 @@ class InfluxDBClient(object):
 
     @staticmethod
     def _batches(iterable, size):
-        for i in xrange(0, len(iterable), size):
-            yield iterable[i:i + size]
+        # Iterate over an iterable producing iterables of batches. Based on:
+        # http://code.activestate.com/recipes/303279-getting-items-in-batches/
+        iterator = iter(iterable)
+        while True:
+            try:  # Try get the first element in the iterator...
+                head = (next(iterator),)
+            except StopIteration:
+                return  # ...so that we can stop if there isn't one
+            # Otherwise, lazily slice the rest of the batch
+            rest = islice(iterator, size - 1)
+            yield chain(head, rest)
 
     def _write_points(self,
                       points,
@@ -616,6 +714,40 @@ class InfluxDBClient(object):
         """
         return list(self.query("SHOW DATABASES").get_points())
 
+    def get_list_series(self, database=None, measurement=None, tags=None):
+        """
+        Query SHOW SERIES returns the distinct series in your database.
+
+        FROM and WHERE clauses are optional.
+
+        :param measurement: Show all series from a measurement
+        :type id: string
+        :param tags: Show all series that match given tags
+        :type id: dict
+        :param database: the database from which the series should be
+            shows, defaults to client's current database
+        :type database: str
+        """
+        database = database or self._database
+        query_str = 'SHOW SERIES'
+
+        if measurement:
+            query_str += ' FROM "{0}"'.format(measurement)
+
+        if tags:
+            query_str += ' WHERE ' + ' and '.join(["{0}='{1}'".format(k, v)
+                                                   for k, v in tags.items()])
+
+        return list(
+            itertools.chain.from_iterable(
+                [
+                    x.values()
+                    for x in (self.query(query_str, database=database)
+                              .get_points())
+                ]
+            )
+        )
+
     def create_database(self, dbname):
         """Create a new database in InfluxDB.
 
@@ -740,7 +872,7 @@ class InfluxDBClient(object):
         query_string = (
             "ALTER RETENTION POLICY {0} ON {1}"
         ).format(quote_ident(name),
-                 quote_ident(database or self._database), shard_duration)
+                 quote_ident(database or self._database))
         if duration:
             query_string += " DURATION {0}".format(duration)
         if shard_duration:
@@ -838,7 +970,7 @@ class InfluxDBClient(object):
         :param username: the username to drop
         :type username: str
         """
-        text = "DROP USER {0}".format(quote_ident(username), method="POST")
+        text = "DROP USER {0}".format(quote_ident(username))
         self.query(text, method="POST")
 
     def set_user_password(self, username, password):
@@ -1119,3 +1251,25 @@ def _parse_netloc(netloc):
             'password': info.password or None,
             'host': info.hostname or 'localhost',
             'port': info.port or 8086}
+
+
+def _msgpack_parse_hook(code, data):
+    if code == 5:
+        (epoch_s, epoch_ns) = struct.unpack(">QI", data)
+        timestamp = datetime.datetime.utcfromtimestamp(epoch_s)
+        timestamp += datetime.timedelta(microseconds=(epoch_ns / 1000))
+        return timestamp.isoformat() + 'Z'
+    return msgpack.ExtType(code, data)
+
+
+class _SocketOptionsAdapter(HTTPAdapter):
+    """_SocketOptionsAdapter injects socket_options into HTTP Adapter."""
+
+    def __init__(self, *args, **kwargs):
+        self.socket_options = kwargs.pop("socket_options", None)
+        super(_SocketOptionsAdapter, self).__init__(*args, **kwargs)
+
+    def init_poolmanager(self, *args, **kwargs):
+        if self.socket_options is not None:
+            kwargs["socket_options"] = self.socket_options
+        super(_SocketOptionsAdapter, self).init_poolmanager(*args, **kwargs)
