@@ -33,6 +33,7 @@ import requests.exceptions
 import requests_mock
 
 from nose.tools import raises
+from urllib3.connection import HTTPConnection
 
 from influxdb import InfluxDBClient
 from influxdb.resultset import ResultSet
@@ -1398,7 +1399,7 @@ class TestInfluxDBClient(unittest.TestCase):
             InfluxDBClient('host', '80/redir', 'username', 'password')
 
     def test_chunked_response(self):
-        """Test chunked reponse for TestInfluxDBClient object."""
+        """Test chunked response for TestInfluxDBClient object."""
         example_response = \
             u'{"results":[{"statement_id":0,"series":[{"columns":["key"],' \
             '"values":[["cpu"],["memory"],["iops"],["network"]],"partial":' \
@@ -1498,6 +1499,40 @@ class TestInfluxDBClient(unittest.TestCase):
             self.assertEqual(m.last_request.headers["Authorization"],
                              "my-token")
 
+    def test_custom_socket_options(self):
+        """Test custom socket options."""
+        test_socket_options = HTTPConnection.default_socket_options + \
+            [(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1),
+             (socket.IPPROTO_TCP, socket.TCP_KEEPINTVL, 60),
+             (socket.IPPROTO_TCP, socket.TCP_KEEPCNT, 15)]
+
+        cli = InfluxDBClient(username=None, password=None,
+                             socket_options=test_socket_options)
+
+        self.assertEquals(cli._session.adapters.get("http://").socket_options,
+                          test_socket_options)
+        self.assertEquals(cli._session.adapters.get("http://").poolmanager.
+                          connection_pool_kw.get("socket_options"),
+                          test_socket_options)
+
+        connection_pool = cli._session.adapters.get("http://").poolmanager \
+            .connection_from_url(
+            url="http://localhost:8086")
+        new_connection = connection_pool._new_conn()
+        self.assertEquals(new_connection.socket_options, test_socket_options)
+
+    def test_none_socket_options(self):
+        """Test default socket options."""
+        cli = InfluxDBClient(username=None, password=None)
+        self.assertEquals(cli._session.adapters.get("http://").socket_options,
+                          None)
+        connection_pool = cli._session.adapters.get("http://").poolmanager \
+            .connection_from_url(
+            url="http://localhost:8086")
+        new_connection = connection_pool._new_conn()
+        self.assertEquals(new_connection.socket_options,
+                          HTTPConnection.default_socket_options)
+
 
 class FakeClient(InfluxDBClient):
     """Set up a fake client instance of InfluxDBClient."""
